package givicenc;
import iic2132.*;

/**
 * @author givicenc / Germn I. Vicencio
 *
 */
public class MyMap {

	private Position[][] map;

	//----------------------------------------------------------------------------
	
	public Position CurrentPosition;
	
	//----------------------------------------------------------------------------
	
	public MyMap(int length) 
	{
		map = new Position[length][length];
		
		// Inicializar cada celda como un nuevo Position (visits = 0, content = UNKNOWN)
		for (int i=0; i < map.length ; i++) 
			for (int j=0; j < map.length; j++) 
			{
				// Nuevo nodo ahi
				map[i][j] = new Position(i, j);
				
			}

		
		// Pto inicial
		CurrentPosition = GetPositionAt((length-1)/2 , (length-1)/2);
		
	}
	
	//----------------------------------------------------------------------------
	
	public int CurrentRow() {
		return CurrentPosition.Row;
	}
	
	//----------------------------------------------------------------------------
	
	public int CurrentCol() {
		return CurrentPosition.Col;
	}
	
	//----------------------------------------------------------------------------
	
	/**
	 *  Si no son indices validos devuelve null.   <br>
	 *  Si son validos devuelve el Position ahi,   <br>
	 *  ie. map[x][y]
	 *  
	 **/
	public Position GetPositionAt(int x, int y) 
	{
		if (x < 0 || y < 0 || x >= map.length || y >= map.length)
		{
			return null;
		}
		
		return map[x][y];
	}
	
	//----------------------------------------------------------------------------
	
	/**
	 *  Actualiza currentPosition y le agrega 1 visit  <br>
	 *  Retorna direction
	 *  
	 **/
	public int MoveAndLog(int direction) 
	{
		
		// Update currentPosition
		CurrentPosition = GetPosition(direction);
		CurrentPosition.Visits++;
		
		// Finalmente retornar
		return direction;
	}
	
	//----------------------------------------------------------------------------

	/**
	 *  Desde start se va moviendo en esa direccion <br>
	 *  una posicion a la vez y la va marcando      <br>
	 *  con IsGoalPath = true.
	 *  
	 **/
	public void MarkGoalPath(Position start, int direction)
	{
		Position temp = start;
		
	
		// Ir moviendose en esa direccion y construyendo el path
		while (temp != null) {
			temp = GetPosition(temp, direction);
			
			if (temp != null && temp.Unvisited()) 
			{
				temp.IsInGoalPath = true;	// Pintar
			}
		}
		
		
		
	}
	
	//----------------------------------------------------------------------------
	
	/**
	 *  A todas las celdas de la matriz les deja IsInGoalPath = false.
	 *  
	 **/
	public void ClearGoalPath()
	{
		// Para cada celda...
		for (int i=0; i < map.length ; i++) 
			for (int j=0; j < map.length; j++) 

				// Desmarcar
				map[i][j].IsInGoalPath = false;
	}
	
	//----------------------------------------------------------------------------
	
	/**
	 *  Devuelve la posicion que est moviendose         <br>
	 *  una celda en esa direccion desde CurrentPosition 
	 *  
	 **/
	public Position GetPosition(int direction) 
	{
		
		return GetPosition( CurrentPosition, direction );
	}
	
	//----------------------------------------------------------------------------
	
	/**
	 *  Devuelve la posicion que est moviendose         <br>
	 *  una celda en esa direccion desde from 
	 *  
	 **/
	public Position GetPosition(Position from, int direction) 
	{

		switch (direction) 
		{
			case Directions.E:
				return GetPositionAt(from.Row  , from.Col+1);
			case Directions.NE:
				return GetPositionAt(from.Row-1, from.Col+1); 
			case Directions.N:
				return GetPositionAt(from.Row-1, from.Col);
			case Directions.NW:
				return GetPositionAt(from.Row-1, from.Col-1);
			case Directions.W:
				return GetPositionAt(from.Row  , from.Col-1);
			case Directions.SW:
				return GetPositionAt(from.Row+1, from.Col-1);
			case Directions.S:
				return GetPositionAt(from.Row+1, from.Col);
			case Directions.SE:
				return GetPositionAt(from.Row+1, from.Col+1);
		}
		
		return null;
		
	}
	
	//----------------------------------------------------------------------------
}
