package givicenc;
//import java.util.*;
import java.util.Random;

import iic2132.*;

/**
 * @author givicenc / Germn I. Vicencio <br><br>
 *
 * 
 */
public class MyNavigator implements INavigator {
	
	//----------------------------------------------------------------------------
	
	private MyMap myMap;
	private int myState;                          // RobotStates.WALLFOLLOW vs .GOALSEEK
	private int currentRobotOrientation = -1;     // Pa donde esta mirando el robot
	private int currentWallFollowMode = -1;	      // WallFollowModes.LEFT vs .RIGHT
	
	private final int wallFollowFirstTryTolerance = 300; // Tolerancia de alejamiento (en pasos)
	private boolean secondWallFollowTry = false;        // Cuando se arrepiente de su decisin
	private int currentMovingAwayCounter = 0;			// Conteo de pasos
	
	//----------------------------------------------------------------------------

	public MyNavigator() 
	{
		myMap = new MyMap(401);
		
		
		// Pto inicial donde est parado est vaco
		myMap.CurrentPosition.SetContent( World.EMPTY );
		myMap.CurrentPosition.Visits = 1;
		
		// Objetivo inicial:
		myState = RobotStates.GOALSEEK;
			
	}

	//----------------------------------------------------------------------------
	
	/**
	 * Primero actualiza su mapa personal:                              <br>
	 * -Agrega los 8 adyacentes al mapa,								<br>
	 * -Agrupa obstaculos adyacentes como si fueran un solo obstaculo.  <br><br>
	 *  
	 * Luego busca en los adyacentes si est el goal.					<br>
	 * Si est, termina.                                                <br><br>
	 * 
	 * Luego hace lo que debe hacer dependiendo de su estado:           <br>
	 * GOALSEEK vs OBSTACLEFOLLOW                                       <br><br>
	 *  
	 * En cada uno siempre hay una verificacion para ver si se debe     <br><br>
	 * cambiar de ese estado al otro.									<br>
	 * 
	 */
	@Override
	public int navigate(char[] surroundings, int goalDirection) 
	{	
		// Actualizar conocimiento: Punto donde estoy + Surroundings
		UpdateKnowledge(surroundings, goalDirection);
		
		
		// Goal es adyacente:
		if (GoalIsAdjacent()) return goalDirection;

		
		// Modo perseguir goal
		if (myState == RobotStates.GOALSEEK)
		{
			return myMap.MoveAndLog( DoGoalSeek() );
		}

		
		// Modo rodear obstaculo
		if (myState == RobotStates.WALLFOLLOW)
		{
			return myMap.MoveAndLog( DoWallFollow() );
		}
		
		
		return -1; //dummy

	}
	
	//----------------------------------------------------------------------------
	
	private int DoGoalSeek() 
	{
		int goalDirection = myMap.CurrentPosition.GoalDirection;
		
		
		// Si en direccion al goal hay obstaculo: Cambiar estado
		if (myMap.GetPosition( goalDirection ).IsObstacle()) 
		{
			myState = RobotStates.WALLFOLLOW;
			MakeHandTouchWall();
			
			// Reiniciar sistema de tolerancia
			secondWallFollowTry = false;
			currentMovingAwayCounter = 0;
			
			// Ahora que entr a este state, dibujar path desde ahi.
			myMap.ClearGoalPath();
			myMap.MarkGoalPath(  myMap.CurrentPosition, goalDirection  );
			
			return DoWallFollow();
			
		}
		// No hay obstaculo, perseguir goal.
		else 
		{
			return goalDirection;
		}
	}
	
	//----------------------------------------------------------------------------
	
	private int DoWallFollow()
	{
		// Condicion para DETENER el wall following
		if ( GoalPathIsAdjacent() ) // Se top con la linea hacia el goal denuevo
		{
			
			myState = RobotStates.GOALSEEK;
			
			// Seguir persiguiendo goal
			return myMap.CurrentPosition.GoalDirection;
			
		}
		
		// Super tolerancia, cambiar modo y empezar 2nd try (y no pescar esto si ya est en el 2nd try)
		ToleranceCounter();
		if (currentMovingAwayCounter >= wallFollowFirstTryTolerance && !secondWallFollowTry)
		{
			secondWallFollowTry = true;
			
			InvertWallFollowMode();
		}
		
		// Modo rodear por la derecha
		if (currentWallFollowMode == WallFollowModes.RIGHT) 
		{
			return DoRightWallFollow();
		}
		
		// Modo rodear por la izquierda
		if (currentWallFollowMode == WallFollowModes.LEFT) 
		{
			return DoLeftWallFollow();
		}
		
		// dummy
		return -1;
	}
	
	//----------------------------------------------------------------------------
	
	private void ToleranceCounter() 
	{
		int goalDirection = myMap.CurrentPosition.GoalDirection;

		// Si se est alejando, incrementar
		if (goalDirection == AbsoluteDirectionToBack()           ||
			goalDirection == AbsoluteDirectionTo135DegreesLeft() || 
			goalDirection == AbsoluteDirectionTo135DegreesRight())
		{
			currentMovingAwayCounter++;
		}
		// Si no se esta alejando, reiniciar contador
		else
		{
			currentMovingAwayCounter = 0;
		}
		
	}
	
	//----------------------------------------------------------------------------
	
	private void InvertWallFollowMode() 
	{
		//if (myState != RobotStates.WALLFOLLOW) 
			//Excepcion!!
		
		// Entonces se puede asumir mano derecha o izquierda en muro (una de las dos)
		// Rotarlo en 180 para que quede la otra mano tocando..
		Turn180Degrees();
		
		if (currentWallFollowMode == WallFollowModes.RIGHT) 
		{
			currentWallFollowMode = WallFollowModes.LEFT;
			return;
		}
		else 
		{
			currentWallFollowMode = WallFollowModes.RIGHT;
			return;
		}

	}
	
	//----------------------------------------------------------------------------
	
	private int DoRightWallFollow() 
	{
		// Parto de la base que mi mano izq toca el muro
		if (ContentAtMyLeft() == World.OBSTACLE)
		{
			// Esta orientado ortogonalmente
			if (currentRobotOrientation == Directions.N || currentRobotOrientation == Directions.S || currentRobotOrientation == Directions.W || currentRobotOrientation == Directions.E )
			{
				if (ContentAt45DegreesLeft() != World.OBSTACLE)
				{
					int AbsoluteDirectionTo45DegreesLeft = AbsoluteDirectionTo45DegreesLeft();
					TurnLeft90Degrees();
					return AbsoluteDirectionTo45DegreesLeft;
				}
				else 
				{
					if (ContentAtMyFront() == World.OBSTACLE) 
					{
						TurnRight90Degrees();
						return DoWallFollow();
					}
					else 
					{
						return AbsoluteDirectionToFront();
					}
				}
			}
			// No esta orientado ortogonalmente
			else 
			{
				if (ContentAt45DegreesLeft() != World.OBSTACLE)
				{
					int AbsoluteDirectionTo45DegreesLeft = AbsoluteDirectionTo45DegreesLeft();
					TurnLeft45Degrees();
					return AbsoluteDirectionTo45DegreesLeft;
				}
				else 
				{
					TurnRight45Degrees();
					return DoWallFollow();
				}
			}

		}
		else 
		{
			//throw new Exception
			//("No deberia estar corriendo el algoritmo sin orientar el mono antes.");	
			
		}
			
		
		// dummy.
		return -1;
	}
	
	//----------------------------------------------------------------------------
	
	private int DoLeftWallFollow() 
	{
		// Parto de la base que mi mano derecha toca el muro
		if (ContentAtMyRight() == World.OBSTACLE)
		{
			// Esta orientado ortogonalmente
			if (currentRobotOrientation == Directions.N || currentRobotOrientation == Directions.S || currentRobotOrientation == Directions.W || currentRobotOrientation == Directions.E )
			{
				if (ContentAt45DegreesRight() != World.OBSTACLE)
				{
					int AbsoluteDirectionTo45DegreesRight = AbsoluteDirectionTo45DegreesRight();
					TurnRight90Degrees();
					return AbsoluteDirectionTo45DegreesRight;
				}
				else 
				{
					if (ContentAtMyFront() == World.OBSTACLE) 
					{
						TurnLeft90Degrees();
						return DoWallFollow();
					}
					else 
					{
						return AbsoluteDirectionToFront();
					}
				}
			}
			// No esta orientado ortogonalmente
			else 
			{
				if (ContentAt45DegreesRight() != World.OBSTACLE)
				{
					int AbsoluteDirectionTo45DegreesRight = AbsoluteDirectionTo45DegreesRight();
					TurnRight45Degrees();
					return AbsoluteDirectionTo45DegreesRight;
				}
				else 
				{
					TurnLeft45Degrees();
					return DoWallFollow();
				}
			}

		}
		else 
		{
			//throw new Exception
			//("No deberia estar corriendo el algoritmo sin orientar el mono antes.");	
			
		}
			
		
		// dummy.
		return -1;
	}
	
	//----------------------------------------------------------------------------
	
	private void MakeHandTouchWall() 
	{
		// Parte MIRANDO al goal:
		currentRobotOrientation = myMap.CurrentPosition.GoalDirection;
		
		// Aca decide como va rodear la muralla...
		if (GoalIsToTheLeft())
		{
			currentWallFollowMode = WallFollowModes.LEFT;
			TurnLeft90Degrees(); // Girar para quedar con mano derecha en muro.
			return;
		}
		
		else if (GoalIsToTheRight())
		{
			currentWallFollowMode = WallFollowModes.RIGHT;
			TurnRight90Degrees(); // Girar para quedar con mano izquierda en muro
			return;
		}
		
		else // Hacerlo random algun dia..
		{
			Random rnd = new Random();
			
			if (rnd.nextBoolean()) 
			{
				currentWallFollowMode = WallFollowModes.LEFT;
				TurnLeft90Degrees(); // Girar para quedar con mano derecha en muro.
			}
			else 
			{
				currentWallFollowMode = WallFollowModes.RIGHT;
				TurnRight90Degrees(); // Girar para quedar con mano izquierda en muro
			}
			
		}

	}
	
	//----------------------------------------------------------------------------

	private boolean GoalIsToTheLeft() 
	{
		int goalDirection = myMap.CurrentPosition.GoalDirection;
		
		return (goalDirection == Directions.NW ||
				goalDirection == Directions.SE);
		
	}
	
	//----------------------------------------------------------------------------

	private boolean GoalIsToTheRight() 
	{
		int goalDirection = myMap.CurrentPosition.GoalDirection;
		
		return (goalDirection == Directions.NE ||
				goalDirection == Directions.SW);
		
	}
	
	//----------------------------------------------------------------------------
	
	private void TurnRight90Degrees() 
	{
		int result = currentRobotOrientation - 2;
		
		if (result == -1) result = 7;
		if (result == -2) result = 6;
		
		currentRobotOrientation = result;
	}
	
	//----------------------------------------------------------------------------
	
	private void TurnLeft90Degrees() 
	{
		int result = currentRobotOrientation + 2;
		
		if (result == 8) result = 0;
		if (result == 9) result = 1;
		
		currentRobotOrientation = result;
	}
	
	//----------------------------------------------------------------------------
	
	private void TurnLeft45Degrees() 
	{
		int result = currentRobotOrientation + 1;
		
		if (result == 8) result = 0;
		
		currentRobotOrientation = result;
	}
	
	//----------------------------------------------------------------------------
	
	private void TurnRight45Degrees() 
	{
		int result = currentRobotOrientation - 1;
		
		if (result == -1) result = 7;
		
		currentRobotOrientation = result;
	}
	
	//----------------------------------------------------------------------------
	
	private void Turn180Degrees() 
	{	
		currentRobotOrientation = AbsoluteDirectionToBack();
	}
	
	//----------------------------------------------------------------------------

	private char ContentAtMyLeft() 
	{
		int result = currentRobotOrientation + 2;
		
		if (result == 8) result = 0;
		if (result == 9) result = 1;
		
		return myMap.GetPosition(result).GetContent();
	}
	
	//----------------------------------------------------------------------------

	private char ContentAtMyRight() 
	{
		int result = currentRobotOrientation - 2;
		
		if (result == -1) result = 7;
		if (result == -2) result = 6;
		
		return myMap.GetPosition(result).GetContent();
	}
	
	//----------------------------------------------------------------------------

	private char ContentAt45DegreesLeft() 
	{
		int result = currentRobotOrientation + 1;
		
		if (result == 8) result = 0;
		
		return myMap.GetPosition(result).GetContent();
	}
	
	//----------------------------------------------------------------------------

	private char ContentAt45DegreesRight() 
	{
		int result = currentRobotOrientation - 1;
		
		if (result == -1) result = 7;
		
		return myMap.GetPosition(result).GetContent();
	}
	
	//----------------------------------------------------------------------------

	private char ContentAtMyFront() 
	{
		return myMap.GetPosition(currentRobotOrientation).GetContent();
	}
	
	//----------------------------------------------------------------------------
	
	private int AbsoluteDirectionTo45DegreesLeft() 
	{
		int result = currentRobotOrientation + 1;
		
		if (result == 8) result = 0;
		
		return result;
	}
	
	//----------------------------------------------------------------------------
	
	private int AbsoluteDirectionTo45DegreesRight() 
	{
		int result = currentRobotOrientation - 1;
		
		if (result == -1) result = 7;
		
		return result;
	}
	
	//----------------------------------------------------------------------------
	
	private int AbsoluteDirectionToFront() 
	{
		return currentRobotOrientation;
	}
	
	//----------------------------------------------------------------------------
	
	private int AbsoluteDirectionToBack() 
	{
		int result = currentRobotOrientation - 4;
		
		if (result == -1) result = 7;
		if (result == -2) result = 6;
		if (result == -3) result = 5;
		if (result == -4) result = 4;
		
		return result;
	}
	
	//----------------------------------------------------------------------------
	
	private int AbsoluteDirectionTo135DegreesLeft() // Diagonal left back
	{
		int result = currentRobotOrientation + 3;
		
		if (result == 8) result = 0;
		if (result == 9) result = 1;
		if (result == 10) result = 2;
		
		return result;
	}
	
	//----------------------------------------------------------------------------
	
	private int AbsoluteDirectionTo135DegreesRight() // Diagonal right back
	{
		int result = currentRobotOrientation - 3;
		
		if (result == -1) result = 7;
		if (result == -2) result = 6;
		if (result == -3) result = 5;
		
		return result;
	}
	
	//----------------------------------------------------------------------------
	
	private boolean GoalIsAdjacent() 
	{
		int goalDirection = myMap.CurrentPosition.GoalDirection;
		
		return myMap.GetPosition( goalDirection ).GetContent() == World.GOAL ;
				
	}
	
	//----------------------------------------------------------------------------
	
	private boolean GoalPathIsAdjacent() 
	{
		// Va viendo si cualquiera de los adjacents es goal path (y no obstaculo)
		for (int i=-1; i<=1; i++) {
			for (int j=-1; j<=1; j++) {
				
				Position p = myMap.GetPositionAt( myMap.CurrentRow()+i , myMap.CurrentCol()+j );
				if (p != null)
				{
					if (p.IsInGoalPath && !p.IsObstacle()) {
						return true;
					}
				}
			}
		}
		
		return false;
	}
	
	//----------------------------------------------------------------------------
	
	private void UpdateKnowledge(char[] surroundings, int goalDirection) 
	{
		// Here
		myMap.CurrentPosition.SetContent( World.EMPTY ); //si estoy parado arriba es empty.
		myMap.CurrentPosition.GoalDirection = goalDirection;
		
		// Surroundings
		for (int i=0; i <= 7; i++) {
			myMap.GetPosition(i).SetContent( surroundings[i] );
		}

	}

	//---------------------------------------------------------------------------

}
