package iic2132;

import javax.swing.*;
import java.awt.*;

/**
 * @author dimatura
 *
 */
public class GSimulator {
	static final int DEFAULT_DX = 13;
	static final int DEFAULT_DY = 13;
	INavigator navigator;
	World maze;
	GSimPanel gsimpanel;
	
	class GSimPanel extends JPanel {
		private static final long serialVersionUID = 1L;

		public void paintComponent(Graphics g) {		
			int dx = this.getWidth()/maze.getCols();
			int dy = this.getHeight()/maze.getRows();
			int horiz_length = maze.getCols()*dx-1;
			int vert_length = maze.getRows()*dy-1;
			int x = 0, y = 0;			
			// draw horizontal lines
			g.setColor(Color.gray);
			for (int i=0; i < maze.getRows(); ++i) {
				g.drawLine(0, y, horiz_length, y);
				//g.drawLine(0, y, length, y);
				y += dy;
			}
			// draw vertical grid lines
            for (int j = 0; j < maze.getCols(); ++j) {
                g.drawLine(x, 0, x, vert_length);
                x += dx;
            }
            
            y = 0;
            for (int i = 0; i < maze.getRows(); ++i) {
                x = 0;
                for (int j = 0; j < maze.getCols(); ++j) {
                	char c = maze.getMazeData(i, j);
                	switch (c) {
                	case World.OBSTACLE:
                		g.setColor(Color.black);
                		g.fillRect(x, y, dx+1, dy+1);
                		break;
                	case World.PATH:
                		g.setColor(Color.lightGray);
                		g.fillRect(x+1, y+1, dx-1, dy-1);
                		//g.fillRect(x, y, dx, dy);
                		break;
                	case World.GOAL:
                		g.setColor(Color.red);
                		//g.drawString("G", x, y);
                		g.fillOval(x+1, y+1, dx-2, dy-2);
                		break;
                	case World.ROBOT:
                   		g.setColor(Color.blue);
                		g.fillOval(x+1, y+1, dx-2, dy-2);
                		break;                	
                	}                    
                    x += dx;
                }
                y += dy;
            }			
		}
	}
	
	public GSimulator(INavigator navigator, World maze) {
		this.navigator = navigator;
		this.maze = maze;
	}
	
	void initGui() {
		JFrame frame = new JFrame("EPICROBOTSIM!!1!1");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		gsimpanel = new GSimPanel();
		frame.getContentPane().add(gsimpanel);
		frame.setSize(DEFAULT_DX*maze.getCols(), DEFAULT_DY*maze.getRows());
		frame.setVisible(true);
		
	}
	
	public double go(int maxSteps, int delayMs) {
		initGui();
		int steps;
		double pathLength = 0.0;
		for (steps=0; steps < maxSteps; ++steps) {
			char s[] = maze.surroundings();
			int goaldir = maze.goalDirection();
			int direction = navigator.navigate(s, goaldir);
			if (direction == Directions.NE || direction == Directions.NW
					|| direction == Directions.SE || direction == Directions.SW) {
				pathLength += Math.sqrt(2.0);
			} else {
				pathLength += 1;
			}
			maze.moveRobot(direction);
			gsimpanel.repaint();
			// delay for stupid slow human mind
			try {
				Thread.sleep(delayMs);
			} catch (Exception ex) {
				// whatever
			}
			if (maze.robotAtGoal()) {
				return pathLength;
			}			
		}
		return pathLength;
	}


}
