package iic2132;

import java.io.PrintStream;

/**
 * @author dimatura
 *
 */
public class World {
	public static final char OBSTACLE = '#';
	public static final char EMPTY = '.';
	public static final char ROBOT = '*';
	public static final char GOAL = 'G';
	public static final char PATH = 'o';
	public static final char UNKNOWN = 'U';
	char mapData[][];
	int rows;
	int cols;
	Position goalPos = null;
	Position robotPos = null;

	World(char mazeData[][], Position startPos, Position goalPos) {
		this.mapData = mazeData;
		this.rows = mazeData.length;
		this.cols = mazeData[0].length;
		this.robotPos = startPos;
		this.goalPos = goalPos;
	}

	void putRobot(int r, int c) {
		robotPos.row = r;
		robotPos.col = c;
	}

	void moveRobot(int direction) {
		Position newpos = null;
		int r = robotPos.row, c = robotPos.col;
		switch (direction) {
		case Directions.E:
			newpos = new Position(r, c + 1);
			break;
		case Directions.NE:
			newpos = new Position(r - 1, c + 1);
			break;
		case Directions.N:
			newpos = new Position(r - 1, c);
			break;
		case Directions.NW:
			newpos = new Position(r - 1, c - 1);
			break;
		case Directions.W:
			newpos = new Position(r, c - 1);
			break;
		case Directions.SW:
			newpos = new Position(r + 1, c - 1);
			break;
		case Directions.S:
			newpos = new Position(r + 1, c);
			break;
		case Directions.SE:
			newpos = new Position(r + 1, c + 1);
			break;
		default:
			throw new RuntimeException("Direction does not exist.");
		}
		if (mapData[newpos.row][newpos.col] != World.OBSTACLE) {
			mapData[robotPos.row][robotPos.col] = World.PATH;
			mapData[newpos.row][newpos.col] = World.ROBOT;
			robotPos = newpos;
		}
		// else don't move
	}

	boolean robotAtGoal() {
		return (robotPos.col == goalPos.col && robotPos.row == goalPos.row);
	}

	char[] surroundings() {
		int r = robotPos.row, c = robotPos.col;
		char s[] = new char[8];
		s[Directions.E] = mapData[r][c + 1];
		s[Directions.NE] = mapData[r - 1][c + 1];
		s[Directions.N] = mapData[r - 1][c];
		s[Directions.NW] = mapData[r - 1][c - 1];
		s[Directions.W] = mapData[r][c - 1];
		s[Directions.SW] = mapData[r + 1][c - 1];
		s[Directions.S] = mapData[r + 1][c];
		s[Directions.SE] = mapData[r + 1][c + 1];
		for (int i=0; i<s.length; ++i) {
			if (s[i]==World.PATH) {
				s[i] = World.EMPTY;
			}
		}
		return s;
	}

	int goalDirection() {

		double dx = goalPos.col - robotPos.col;
		double dy = -goalPos.row + robotPos.row;
		double angle = Math.toDegrees(Math.atan2(dy, dx) + Math.PI);
		int dir = Directions.W;
		if (angle > 22.5 + 0 * 45)
			dir = Directions.SW;
		if (angle > 22.5 + 1 * 45)
			dir = Directions.S;
		if (angle > 22.5 + 2 * 45)
			dir = Directions.SE;
		if (angle > 22.5 + 3 * 45)
			dir = Directions.E;
		if (angle > 22.5 + 4 * 45)
			dir = Directions.NE;
		if (angle > 22.5 + 5 * 45)
			dir = Directions.N;
		if (angle > 22.5 + 6 * 45)
			dir = Directions.NW;
		if (angle > 22.5 + 7 * 45)
			dir = Directions.W;
		// System.out.println("dx=" + dx + ",dy = " + dy + ", a = " + angle +
		// ", dir = " + dir);
		// System.out.println(dir);
		return dir;
	}

	void print(PrintStream stream) {
		for (int i = 0; i < mapData.length; ++i) {
			for (int j = 0; j < mapData[i].length; ++j) {
				stream.print(mapData[i][j] == World.EMPTY ? ' '
						: mapData[i][j]);
			}
			stream.println();
		}
	}

	int getRows() {
		return rows;
	}

	int getCols() {
		return cols;
	}

	char getMazeData(int r, int c) {
		return mapData[r][c];
	}

}
