package iic2132;

import java.io.*;
import java.util.ArrayList;


/**
 * @author dimatura
 *
 */
public class WorldLoader {
	public WorldLoader() {
		// TODO Auto-generated constructor stub
	}

	public World build(String filename) throws IOException, WorldLoaderException {	
		File file = new File(filename);			
		BufferedReader reader = new BufferedReader(new FileReader(file));
		ArrayList<String> buffer = new ArrayList<String>();
		// read all lines into 'buffer'
		String line = null;
		//int System.getProperty("line.separator"
		while ((line = reader.readLine()) != null) {
			line = line.trim();				
			buffer.add(line);				
		}
		reader.close();
		// check for enough lines
		if (buffer.isEmpty()) {
			throw new WorldLoaderException("Input file was empty.");
		}
		// check all lines have same length
		for (int i=1; i<buffer.size(); ++i) {
			if (buffer.get(i).length() != buffer.get(i-1).length()) {
				throw new WorldLoaderException("Line "+(i+1)+" has different length.");
			}				
		}
		
		int rows = buffer.size();
	    int cols = buffer.get(0).length();
		
		char mazeData[][] = new char[rows+2][cols+2];
		// init with walls around maze
    	for (int i = 0; i < mazeData.length; ++i) {
    		for (int j=0; j < mazeData[i].length; ++j) {
    			mazeData[i][j] = (i==0 || i==mazeData.length-1 
    					|| j==0 || j==mazeData[i].length-1)? World.OBSTACLE : World.EMPTY;
    		}
    	}    		    
		
		// parse map
		Position startPos = null;
		Position goalPos = null;
		for (int i = 0; i < buffer.size(); ++i) {
			for (int j = 0; j < buffer.get(0).length(); ++j) {
				char c = buffer.get(i).charAt(j);
				if (c == World.ROBOT) {
					if (startPos == null) {
						// shift pos in (1,1) to account for wall
						startPos = new Position(i + 1, j + 1);
						mazeData[i+1][j+1] = World.ROBOT;
					} else {
						throw new WorldLoaderException(
								"More than one starting point in map. Second at "
										+ i + ", " + j);
					}
				} else if (c == World.GOAL) {
					if (goalPos == null) {
						goalPos = new Position(i + 1, j + 1);
						mazeData[i+1][j+1] = World.GOAL;
					} else {
						throw new WorldLoaderException(
								"More than one goal in map. Second at " + i
										+ ", " + j);
					}
				} else if (c == World.OBSTACLE) {
					mazeData[i + 1][j + 1] = World.OBSTACLE;
				} else if (c == World.EMPTY) {
					// do nothing
				} else {
					throw new WorldLoaderException("Unknown character at " + i + ", "
							+ j + ":" + c);
				}
			}
		}	
		if (startPos == null) {
			throw new WorldLoaderException("No start position");
		}
		if (goalPos == null) {
			throw new WorldLoaderException("No goal position");			
		}
		return new World(mazeData, startPos, goalPos);
	}

}
